@@ -45,7 +45,7 @@
 #include "hg_lock_n_thread.h"
 #include "hg_addrdescr.h"            /* self */
 
-void HG_(describe_addr) ( Addr a, /*OUT*/AddrInfo* ai )
+void HG_(describe_addr) ( DiEpoch ep, Addr a, /*OUT*/AddrInfo* ai )
 {
    tl_assert(ai->tag == Addr_Undescribed);
 
@@ -81,7 +81,7 @@ void HG_(describe_addr) ( Addr a, /*OUT*/AddrInfo* ai )
       ai->Addr.Block.freed_at = VG_(null_ExeContext)();;
    } else {
       /* No block found. Search a non-heap block description. */
-      VG_(describe_addr) (a, ai);
+      VG_(describe_addr) (ep, a, ai);
 
       /* In case ai contains a tid, set tnr to the corresponding helgrind
          thread number. */
@@ -100,14 +100,14 @@ void HG_(describe_addr) ( Addr a, /*OUT*/AddrInfo* ai )
    }
 }
 
-Bool HG_(get_and_pp_addrdescr) (Addr addr)
+Bool HG_(get_and_pp_addrdescr) (DiEpoch ep, Addr addr)
 {
 
    Bool ret;
    AddrInfo glai;
 
    glai.tag = Addr_Undescribed;
-   HG_(describe_addr) (addr, &glai);
+   HG_(describe_addr) (ep, addr, &glai);
    VG_(pp_addrinfo) (addr, &glai);
    ret = glai.tag != Addr_Unknown;
 
