@@ -434,8 +434,8 @@ void example1b(void)
 
 typedef struct {
    Int   b1;
-   Addr  first;
-   Addr  last;
+   RegWord  first;
+   RegWord  last;
    Int   b2;
 }
 Block;
@@ -445,13 +445,14 @@ static HChar *blockToStr(void *p)
 {
    static HChar buf[32];
    Block* b = (Block*)p;
-   sprintf(buf, "<(%d) %lu..%lu (%d)>", b->b1, b->first, b->last, b->b2);
+   sprintf(buf, "<(%d) %" FMT_REGWORD "u..%" FMT_REGWORD "u (%d)>",
+           b->b1, b->first, b->last, b->b2);
    return buf;
 }
 
 static Word blockCmp(const void* vkey, const void* velem)
 {
-   Addr   key  = *(const Addr*)vkey;
+   RegWord   key  = *(const RegWord*)vkey;
    const Block* elem = (const Block*)velem;
 
    assert(elem->first <= elem->last);
@@ -463,7 +464,7 @@ static Word blockCmp(const void* vkey, const void* velem)
 void example2(void)
 {
    Int i, n;
-   Addr a;
+   RegWord a;
    Block* vs[NN];
    Block v, prev;
    Block *pv;
