@@ -101,6 +101,8 @@
    20400 WCSCHR
    20410 WCSRCHR
    20420 STPNCPY
+   20430 WMEMCHR
+   20440 WCSNLEN
 */
 
 #if defined(VGO_solaris)
@@ -377,7 +379,7 @@ static inline void my_exit ( int x )
       while (m < n && *dst) { m++; dst++; } \
       if (m < n) { \
          /* Fill as far as dst_orig[n-2], then nul-terminate. */ \
-         while (m < n-1 && *src) { m++; *dst++ = *src++; } \
+         while (m+1 < n && *src) { m++; *dst++ = *src++; } \
          *dst = 0; \
       } else { \
          /* No space to copy anything to dst. m == n */ \
@@ -580,7 +582,7 @@ static inline void my_exit ( int x )
       \
       STRLCPY_CHECK_FOR_DSTSIZE_ZERO \
       \
-      while (m < n-1 && *src) { m++; *dst++ = *src++; } \
+      while (m+1 < n && *src) { m++; *dst++ = *src++; } \
       /* m non-nul bytes have now been copied, and m <= n-1. */ \
       /* Check for overlap after copying; all n bytes of dst are relevant, */ \
       /* but only m+1 bytes of src if terminator was found */ \
@@ -962,6 +964,11 @@ static inline void my_exit ( int x )
                while ((s & WM) != 0 && n >= 1) \
                   { *(UChar*)d = *(UChar*)s; s += 1; d += 1; n -= 1; } \
                /* Copy UWords. */ \
+               while (n >= WS * 4) \
+                  { *(UWord*)d = *(UWord*)s; s += WS; d += WS; n -= WS;   \
+                    *(UWord*)d = *(UWord*)s; s += WS; d += WS; n -= WS;   \
+                    *(UWord*)d = *(UWord*)s; s += WS; d += WS; n -= WS;   \
+                    *(UWord*)d = *(UWord*)s; s += WS; d += WS; n -= WS; } \
                while (n >= WS) \
                   { *(UWord*)d = *(UWord*)s; s += WS; d += WS; n -= WS; } \
                if (n == 0) \
@@ -989,6 +996,11 @@ static inline void my_exit ( int x )
                while ((s & WM) != 0 && n >= 1) \
                   { s -= 1; d -= 1; *(UChar*)d = *(UChar*)s; n -= 1; } \
                /* Copy UWords. */ \
+               while (n >= WS * 4) \
+                  { s -= WS; d -= WS; *(UWord*)d = *(UWord*)s; n -= WS;   \
+                    s -= WS; d -= WS; *(UWord*)d = *(UWord*)s; n -= WS;   \
+                    s -= WS; d -= WS; *(UWord*)d = *(UWord*)s; n -= WS;   \
+                    s -= WS; d -= WS; *(UWord*)d = *(UWord*)s; n -= WS; } \
                while (n >= WS) \
                   { s -= WS; d -= WS; *(UWord*)d = *(UWord*)s; n -= WS; } \
                if (n == 0) \
@@ -1750,7 +1762,7 @@ static inline void my_exit ( int x )
       UWord len = 0; \
       while (1) { \
          UWord i; \
-         HChar sc = *s; \
+         UChar sc = *s; \
          if (sc == 0) \
             break; \
          for (i = 0; i < nacc; i++) { \
@@ -1843,9 +1855,9 @@ static inline void my_exit ( int x )
 
 #define WCSLEN(soname, fnname) \
    SizeT VG_REPLACE_FUNCTION_EZU(20370,soname,fnname) \
-      ( const UInt* str ); \
+      ( const Int* str ); \
    SizeT VG_REPLACE_FUNCTION_EZU(20370,soname,fnname) \
-      ( const UInt* str )  \
+      ( const Int* str )  \
    { \
       SizeT i = 0; \
       while (str[i] != 0) i++; \
@@ -1862,6 +1874,28 @@ static inline void my_exit ( int x )
 
 #endif
 
+/*---------------------- wcsnlen ----------------------*/
+
+#define WCSNLEN(soname, fnname) \
+   SizeT VG_REPLACE_FUNCTION_EZU(20440,soname,fnname) \
+            ( const Int *s, SizeT n ); \
+   SizeT VG_REPLACE_FUNCTION_EZU(20440,soname,fnname) \
+            ( const Int *s, SizeT n ) \
+   {                                     \
+      SizeT i = 0;                       \
+      const Int* p = s;                  \
+      while (i < n && *p != 0) {         \
+         i++;                            \
+         p++;                            \
+      }                                  \
+      return i;                          \
+   }
+
+#if defined(VGO_linux)
+ WCSNLEN(VG_Z_LIBC_SONAME, wcsnlen)
+ WCSNLEN(VG_Z_LIBC_SONAME, __GI_wcsnlen)
+#endif
+
 /*---------------------- wcscmp ----------------------*/
 
 // This is a wchar_t equivalent to strcmp.  We don't
@@ -1971,6 +2005,32 @@ static inline void my_exit ( int x )
  WCSRCHR(VG_Z_LIBC_SONAME, wcsrchr)
 #endif
 
+ /*---------------------- wmemchr ----------------------*/
+
+// This is a wchar_t equivalent to memchr.  We don't
+// have wchar_t available here, but in the GNU C Library
+// wchar_t is always 32 bits wide.
+
+#define WMEMCHR(soname, fnname)                        \
+   Int* VG_REPLACE_FUNCTION_EZU(20430,soname,fnname) \
+            (const Int *s, Int c, SizeT n); \
+   Int* VG_REPLACE_FUNCTION_EZU(20430,soname,fnname) \
+            (const Int *s, Int c, SizeT n) \
+   { \
+      SizeT i; \
+      const Int* p = s; \
+      for (i = 0; i < n; i++) {                   \
+         if (*p == c) return CONST_CAST(Int *,p); \
+         p++;                                     \
+      }                                           \
+      return NULL; \
+   }
+
+#if defined(VGO_linux)
+ WMEMCHR(VG_Z_LIBC_SONAME, wmemchr)
+ WMEMCHR(VG_Z_LIBC_SONAME, __GI_wmemchr)
+#endif
+
 /*------------------------------------------------------------*/
 /*--- Improve definedness checking of process environment  ---*/
 /*------------------------------------------------------------*/
